.vertical-key-value {
  display: inline-block;
  line-height: .75;
  text-align: center;
  vertical-align: middle;

  .key {
    font-size: 10/12em;
    color: var(--default-text-color-light, @default-text-color-light);
  }

  .value {
    color: var(--default-text-color, @default-text-color);
    font-size: 1.5em;
    font-weight: bold;
  }
}
