<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'ipl\\Web\\' => array($vendorDir . '/ipl/web/src'),
    'ipl\\Validator\\' => array($vendorDir . '/ipl/validator/src'),
    'ipl\\Stdlib\\' => array($vendorDir . '/ipl/stdlib/src'),
    'ipl\\Sql\\' => array($vendorDir . '/ipl/sql/src'),
    'ipl\\Scheduler\\' => array($vendorDir . '/ipl/scheduler/src'),
    'ipl\\Orm\\' => array($vendorDir . '/ipl/orm/src'),
    'ipl\\I18n\\' => array($vendorDir . '/ipl/i18n/src'),
    'ipl\\Html\\' => array($vendorDir . '/ipl/html/src'),
    'Webmozart\\Assert\\' => array($vendorDir . '/webmozart/assert/src'),
    'Symfony\\Polyfill\\Php84\\' => array($vendorDir . '/symfony/polyfill-php84'),
    'Recurr\\' => array($vendorDir . '/simshaun/recurr/src/Recurr'),
    'React\\Promise\\' => array($vendorDir . '/react/promise/src'),
    'React\\EventLoop\\' => array($vendorDir . '/react/event-loop/src'),
    'Ramsey\\Uuid\\' => array($vendorDir . '/ramsey/uuid/src'),
    'Ramsey\\Collection\\' => array($vendorDir . '/ramsey/collection/src'),
    'Psr\\Log\\' => array($vendorDir . '/psr/log/Psr/Log'),
    'Psr\\Http\\Message\\' => array($vendorDir . '/psr/http-message/src', $vendorDir . '/psr/http-factory/src'),
    'GuzzleHttp\\Psr7\\' => array($vendorDir . '/guzzlehttp/psr7/src'),
    'Evenement\\' => array($vendorDir . '/evenement/evenement/src'),
    'Doctrine\\Deprecations\\' => array($vendorDir . '/doctrine/deprecations/src'),
    'Doctrine\\Common\\Collections\\' => array($vendorDir . '/doctrine/collections/src'),
    'Cron\\' => array($vendorDir . '/dragonmantank/cron-expression/src/Cron'),
    'Brick\\Math\\' => array($vendorDir . '/brick/math/src'),
);
