@font-face {
  font-family: 'Icinga-Icons';
  src:
    url('fonts/Icinga-Icons.ttf?xnw9vw') format('truetype'),
    url('fonts/Icinga-Icons.woff?xnw9vw') format('woff'),
    url('fonts/Icinga-Icons.svg?xnw9vw#Icinga-Icons') format('svg');
  font-weight: normal;
  font-style: normal;
  font-display: block;
}

[class^="iicon-"], [class*=" iicon-"] {
  /* use !important to prevent issues with browser extensions that change fonts */
  font-family: 'Icinga-Icons' !important;
  speak: never;
  font-style: normal;
  font-weight: normal;
  font-variant: normal;
  text-transform: none;
  line-height: 1;

  /* Better Font Rendering =========== */
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.iicon-certificate:before {
  content: "\e906";
}
.iicon-filter-check-circle:before {
  content: "\e90b";
}
.iicon-ca-check-circle:before {
  content: "\e908";
}
.iicon-refresh-cert:before {
  content: "\e909";
}
.iicon-th-list:before {
  content: "\e90a";
}
.iicon-icinga:before {
  content: "\e907";
}
.iicon-list-view-minimal:before {
  content: "\e900";
}
.iicon-list-view-detailed:before {
  content: "\e901";
}
.iicon-list-view-default:before {
  content: "\e902";
}
.iicon-grid:before {
  content: "\e903";
}
.iicon-bracket-open:before {
  content: "\e904";
}
.iicon-bracket-close:before {
  content: "\e905";
}
