<?php return array(
    'root' => array(
        'name' => 'icinga/icinga-php-library',
        'pretty_version' => 'dev-main',
        'version' => 'dev-main',
        'reference' => 'ac844e6464137f2e3b53a663dfb3a6d00599f24b',
        'type' => 'project',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        'brick/math' => array(
            'pretty_version' => '0.13.1',
            'version' => '0.13.1.0',
            'reference' => 'fc7ed316430118cc7836bf45faff18d5dfc8de04',
            'type' => 'library',
            'install_path' => __DIR__ . '/../brick/math',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/collections' => array(
            'pretty_version' => '2.3.0',
            'version' => '2.3.0.0',
            'reference' => '2eb07e5953eed811ce1b309a7478a3b236f2273d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/collections',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/deprecations' => array(
            'pretty_version' => '1.1.5',
            'version' => '1.1.5.0',
            'reference' => '459c2f5dd3d6a4633d3b5f46ee2b1c40f57d3f38',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/deprecations',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dragonmantank/cron-expression' => array(
            'pretty_version' => 'v3.4.0',
            'version' => '3.4.0.0',
            'reference' => '8c784d071debd117328803d86b2097615b457500',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dragonmantank/cron-expression',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'evenement/evenement' => array(
            'pretty_version' => 'v3.0.2',
            'version' => '3.0.2.0',
            'reference' => '0a16b0d71ab13284339abb99d9d2bd813640efbc',
            'type' => 'library',
            'install_path' => __DIR__ . '/../evenement/evenement',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'fortawesome/font-awesome' => array(
            'pretty_version' => '6.7.2',
            'version' => '6.7.2.0',
            'reference' => 'af620534bfc3c2d4cbefcfeec29603bbe7809e64',
            'type' => 'library',
            'install_path' => __DIR__ . '/../fortawesome/font-awesome',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/psr7' => array(
            'pretty_version' => '2.7.1',
            'version' => '2.7.1.0',
            'reference' => 'c2270caaabe631b3b44c85f99e5a04bbb8060d16',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/psr7',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'icinga/icinga-php-library' => array(
            'pretty_version' => 'dev-main',
            'version' => 'dev-main',
            'reference' => 'ac844e6464137f2e3b53a663dfb3a6d00599f24b',
            'type' => 'project',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ipl/html' => array(
            'pretty_version' => 'v0.8.2',
            'version' => '0.8.2.0',
            'reference' => 'e18bdf11abca5e477100e2c7d190ef5f424d0d98',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ipl/html',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ipl/i18n' => array(
            'pretty_version' => 'v0.2.2',
            'version' => '0.2.2.0',
            'reference' => 'a2b6109c5a93f86ce46d5dc351dbe75e8502cf8c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ipl/i18n',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ipl/orm' => array(
            'pretty_version' => 'v0.7.0',
            'version' => '0.7.0.0',
            'reference' => '4893731e82c9a3859e8ea05c4c21d84f067d7cfb',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ipl/orm',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ipl/scheduler' => array(
            'pretty_version' => 'v0.1.2',
            'version' => '0.1.2.0',
            'reference' => '6119afdea07b1390bd728e350e0d80b26ec8d6ba',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ipl/scheduler',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ipl/sql' => array(
            'pretty_version' => 'v0.7.1',
            'version' => '0.7.1.0',
            'reference' => 'e80f1b712c4b96099b0bf9096e6efe317a165e3b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ipl/sql',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ipl/stdlib' => array(
            'pretty_version' => 'v0.14.0',
            'version' => '0.14.0.0',
            'reference' => 'bf5fc8f40b86bd90337db6f3be389be2a93fa64a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ipl/stdlib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ipl/validator' => array(
            'pretty_version' => 'v0.5.0',
            'version' => '0.5.0.0',
            'reference' => 'a601fae0ed330e63cea50e4a2a6659ca1ad97bde',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ipl/validator',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ipl/web' => array(
            'pretty_version' => 'v0.11.1',
            'version' => '0.11.1.0',
            'reference' => 'af3a8a7be5e924f0ad841baf1471d296329aa1e7',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ipl/web',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'mtdowling/cron-expression' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '^1.0',
            ),
        ),
        'psr/http-factory' => array(
            'pretty_version' => '1.1.0',
            'version' => '1.1.0.0',
            'reference' => '2b4765fddfe3b508ac62f829e852b1501d3f6e8a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-factory',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-factory-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-message' => array(
            'pretty_version' => '1.1',
            'version' => '1.1.0.0',
            'reference' => 'cb6ce4845ce34a8ad9e68117c10ee90a29919eba',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-message',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-message-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/log' => array(
            'pretty_version' => '1.1.4',
            'version' => '1.1.4.0',
            'reference' => 'd49695b909c3b7628b6289db5479a1c204601f11',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ralouphie/getallheaders' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '120b605dfeb996808c31b6477290a714d356e822',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ralouphie/getallheaders',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ramsey/collection' => array(
            'pretty_version' => '2.1.1',
            'version' => '2.1.1.0',
            'reference' => '344572933ad0181accbf4ba763e85a0306a8c5e2',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ramsey/collection',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ramsey/uuid' => array(
            'pretty_version' => '4.9.0',
            'version' => '4.9.0.0',
            'reference' => '4e0e23cc785f0724a0e838279a9eb03f28b092a0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ramsey/uuid',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/event-loop' => array(
            'pretty_version' => 'v1.5.0',
            'version' => '1.5.0.0',
            'reference' => 'bbe0bd8c51ffc05ee43f1729087ed3bdf7d53354',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/event-loop',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/promise' => array(
            'pretty_version' => 'v2.11.0',
            'version' => '2.11.0.0',
            'reference' => '1a8460931ea36dc5c76838fec5734d55c88c6831',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/promise',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'rhumsaa/uuid' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '4.9.0',
            ),
        ),
        'simshaun/recurr' => array(
            'pretty_version' => 'v5.0.3',
            'version' => '5.0.3.0',
            'reference' => '7b136768d64f257065e38a804ee6d2f9af6ba6d1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../simshaun/recurr',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php84' => array(
            'pretty_version' => 'v1.32.0',
            'version' => '1.32.0.0',
            'reference' => '000df7860439609837bbe28670b0be15783b7fbf',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php84',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'webmozart/assert' => array(
            'pretty_version' => '1.11.0',
            'version' => '1.11.0.0',
            'reference' => '11cb2199493b2f8a3b53e7f19068fc6aac760991',
            'type' => 'library',
            'install_path' => __DIR__ . '/../webmozart/assert',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'wikimedia/less.php' => array(
            'pretty_version' => 'v3.2.1',
            'version' => '3.2.1.0',
            'reference' => '0d5b30ba792bdbf8991a646fc9c30561b38a5559',
            'type' => 'library',
            'install_path' => __DIR__ . '/../wikimedia/less.php',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
