.empty-state {
  color: @empty-state-color;
}

.empty-state-bar {
  padding: 1em;
  text-align: center;

  .rounded-corners();
  background-color: @empty-state-bar-bg;
  color: @default-text-color;
}
