.horizontal-key-value {
  display: flex;
  padding: .25em 0;
  align-items: baseline;

  .key {
    color: var(--default-text-color-light, @default-text-color-light);
    flex: 0 0 auto;
    white-space: nowrap;
    width: 12em;
  }

  .value {
    color: var(--default-text-color, @default-text-color);
    flex: 1 1 auto;
  }
}
